@@ -1,6 +1,6 @@
 /*
   
-$Header: /cvsroot/tsp/tsp/src/util/libbb/bb_alias.c,v 1.5 2006/11/27 19:55:14 deweerdt Exp $
+$Header: /sources/tsp/tsp/src/util/libbb/bb_alias.c,v 1.7 2007/04/01 13:17:21 deweerdt Exp $
 
 -----------------------------------------------------------------------
 
@@ -52,15 +52,15 @@ Purpose   : Blackboard (alias handling)
 #include "bb_alias.h"
 #include "bb_utils.h"
 
-int32_t 
+int32_t
 bb_isalias(const S_BB_DATADESC_T* data_desc) {
   assert(data_desc);
   return (data_desc->alias_target != -1);
 } /* end of bb_isalias */
 
-int32_t 
-bb_find_aliastack(volatile S_BB_T* bb,       
-		  S_BB_DATADESC_T* data_desc_stack, 
+int32_t
+bb_find_aliastack(volatile S_BB_T* bb,
+		  S_BB_DATADESC_T* data_desc_stack,
 		  int32_t* stack_max_size) {
   int32_t retval             = 0;
   int32_t current_stack_size = 1;
@@ -68,16 +68,16 @@ bb_find_aliastack(volatile S_BB_T* bb,
   assert(bb);
   assert(data_desc_stack);
   assert(stack_max_size);
-  
+
   while ((current_stack_size<(*stack_max_size)) &&
 	 bb_isalias(&data_desc_stack[current_stack_size-1])) {
 
-    data_desc_stack[current_stack_size] = 
+    data_desc_stack[current_stack_size] =
       bb_data_desc(bb)[data_desc_stack[current_stack_size-1].alias_target];
     ++current_stack_size;
   }
   if ((current_stack_size==(*stack_max_size)) && bb_isalias(&data_desc_stack[current_stack_size-1])) {
-    bb_logMsg(BB_LOG_SEVERE,"BlackBoard::bb_find_aliastack", 
+    bb_logMsg(BB_LOG_SEVERE,"BlackBoard::bb_find_aliastack",
 	      "MAX alias stacksize exceeded <%d>",
 	      *stack_max_size);
     retval = -1;
@@ -87,8 +87,8 @@ bb_find_aliastack(volatile S_BB_T* bb,
   return retval;
 } /* end of bb_find_aliastack */
 
-unsigned long 
-bb_aliasstack_offset(S_BB_DATADESC_T* data_desc_stack, 
+unsigned long
+bb_aliasstack_offset(S_BB_DATADESC_T* data_desc_stack,
 		     int32_t* index_stack,
 		     int32_t stack_size) {
   long retval;
@@ -113,6 +113,7 @@ bb_alias_publish(volatile S_BB_T *bb,
   S_BB_DATADESC_T  aliasstack[MAX_ALIAS_LEVEL];
   int32_t          indexstack[MAX_ALIAS_LEVEL];
   int32_t          aliasstack_size = MAX_ALIAS_LEVEL;
+  char *name;
 
   retval = NULL;
   assert(bb);
@@ -126,15 +127,16 @@ bb_alias_publish(volatile S_BB_T *bb,
    * if key already exists.
    */
   bb_lock(bb);
-  if (bb_find(bb,data_desc->name) != -1) {
-     bb_logMsg(BB_LOG_FINER,"BlackBoard::bb_publish",
-	       "Key <%s> already exists in blackboard (automatic subscribe)!!",data_desc->name);
+  name = (char *)bb_get_varname(data_desc);
+  if (bb_find(bb,name) != -1) {
+    char *n = (char *)bb_get_varname(data_desc);
+    bb_logMsg(BB_LOG_FINER,"BlackBoard::bb_publish",
+        "Key <%s> already exists in blackboard (automatic subscribe)!!", n);
+    free(n);
     bb_unlock(bb);
     retval = bb_subscribe(bb,data_desc);
     bb_lock(bb);
-  } 
-  
-  else {
+  } else {
     /* verify available space in BB data descriptor zone */
     if (bb->n_data >= bb->max_data_desc_size) {
       bb_logMsg(BB_LOG_SEVERE,"BlackBoard::bb_alias_publish", 
@@ -143,8 +145,11 @@ bb_alias_publish(volatile S_BB_T *bb,
          
     } 
     else {
+      char *n;
       /* find the target */
-      data_desc->alias_target = bb_find(bb,data_desc_target->name);
+      n = (char *)bb_get_varname(data_desc_target);
+      data_desc->alias_target = bb_find(bb,n);
+      free(n);
       if (-1 != data_desc->alias_target) {
 	      /* 
 	       * check described offset is not
@@ -166,19 +171,21 @@ bb_alias_publish(volatile S_BB_T *bb,
 		            "Cannot resolve alias stack");
 	        }
 	      } else {
+          char *n = (char *)bb_get_varname(data_desc_target);
 	        bb_logMsg(BB_LOG_SEVERE,"BlackBoard::bb_alias_publish", 
 		          "Alias dim * size <%d> * <%d> and offset <%d> goes out of target range <%d> (%s)",
 		          data_desc->dimension,
 		          data_desc->type_size,
 		          data_desc->data_offset,
 		          data_desc_target->type_size,
-		          data_desc_target->name);
+		          n);
 	      }
       } 
       else {
+          char *n = (char *)bb_get_varname(data_desc_target);
 	        bb_logMsg(BB_LOG_SEVERE,"BlackBoard::bb_alias_publish", 
-		          "Target <%s> does not exists",
-		          data_desc_target->name);
+		          "Target <%s> does not exists", n);
+      free(n);
       }
     }
     /* DO NOT initialize publish data zone with default value */
@@ -187,7 +194,8 @@ bb_alias_publish(volatile S_BB_T *bb,
      * structure.
      */
     /* bb_data_initialise(bb,data_desc,NULL); */
-  }    
+  }
+  free(name);
   /* no init in case of automatic subscribe */  
   bb_unlock(bb);  
   return retval;
@@ -201,6 +209,7 @@ bb_alias_subscribe(volatile S_BB_T* bb,
   
   void* retval;
   int32_t  idx;
+  char *n;
   
   retval = NULL;
   assert(bb);
@@ -208,7 +217,9 @@ bb_alias_subscribe(volatile S_BB_T* bb,
 
    /* We seek the data using its key (name) */
   bb_lock(bb);
-  idx = bb_find(bb,data_desc->name);
+  n = (char *)bb_get_varname(data_desc);
+  idx = bb_find(bb, n);
+  free(n);
   if (idx==-1) {
     retval = NULL;      
   } else {
