@@ -1,6 +1,6 @@
 /*
 
-$Header: /cvsroot/tsp/tsp/src/util/libbb/bb_core.c,v 1.27 2006/11/28 13:52:27 erk Exp $
+$Header: /sources/tsp/tsp/src/util/libbb/bb_core.c,v 1.40 2007/04/01 13:17:21 deweerdt Exp $
 
 -----------------------------------------------------------------------
 
@@ -46,6 +46,10 @@ Purpose   : Blackboard Idiom implementation
 #include <unistd.h>
 #include <assert.h>
 #include <ctype.h>
+#include <dlfcn.h>
+#ifndef RTLD_DEFAULT
+#define RTLD_DEFAULT (void *)0
+#endif
 
 #include <tsp_abs_types.h>
 #include <tsp_abs_types.h>
@@ -63,8 +67,8 @@ Purpose   : Blackboard Idiom implementation
  */
 #ifndef __KERNEL__
 static const char* E_BB_2STRING[] = {"DiscoverType",
-				     "double", 
-				     "float", 
+				     "double",
+				     "float",
 				     "int8_t",
 				     "int16_t",
 				     "int32_t",
@@ -81,11 +85,11 @@ static const char* E_BB_2STRING[] = {"DiscoverType",
 
 static const size_t E_BB_TYPE_SIZE[] = {0,
 			#ifndef __KERNEL__
-					sizeof(double), 
-					sizeof(float), 
-                        #else 
-                                        0,
-                                        0,
+					sizeof(double),
+					sizeof(float),
+      #else
+          0,
+          0,
 			#endif
 					sizeof(int8_t),
 					sizeof(int16_t),
@@ -107,14 +111,20 @@ static struct bb_operations sysv_bb_ops;
 extern struct bb_operations k_bb_ops;
 #else
 extern struct bb_operations sysv_bb_ops;
+#if defined(linux) || defined(__linux)
 extern struct bb_operations k_bb_ops;
 #endif
+#endif
 
 /* Note: This is thigtly related to enum bb_type in bb_core.h,
    if you ever modify this, you'll probably need to modify
    enum bb_type */
-static struct bb_operations *ops[] = { 	&sysv_bb_ops,
-					&k_bb_ops };
+static struct bb_operations *ops[] = {
+	&sysv_bb_ops
+#if defined(linux) || defined(__linux)
+	,&k_bb_ops
+#endif
+};
 
 static enum bb_type bb_type(const char *name)
 {
@@ -123,7 +133,7 @@ static enum bb_type bb_type(const char *name)
 #else
 	if (!strncmp(name, "/dev/", 5))
 		return BB_KERNEL;
-	
+
 	return BB_SYSV;
 #endif /* __KERNEL__ */
 }
@@ -181,6 +191,149 @@ bb_type_string2bb_type(const char* bb_type_string) {
   return retval;
 }
 
+#ifdef __KERNEL__
+static inline char *__strdup (const char *s1)
+{
+	char *p;
+	p = (char *)kmalloc((strlen(s1)+1)*sizeof(char), GFP_KERNEL);
+	if (p != NULL)
+		strcpy (p,s1);
+	return p;
+}
+#endif
+static char *bb_get_varname_default(const S_BB_DATADESC_T *dd)
+{
+#ifdef __KERNEL__
+  return __strdup(dd->__name);
+#else
+  return strdup(dd->__name);
+#endif
+}
+
+static int32_t bb_varname_max_len_default(void)
+{
+  return VARNAME_MAX_SIZE;
+}
+
+static int32_t bb_set_varname_default(S_BB_DATADESC_T *dd, const char *key)
+{
+  strncpy(dd->__name, key, VARNAME_MAX_SIZE);
+  return BB_OK;
+}
+
+bb_get_varname_fn bb_get_varname = bb_get_varname_default;
+bb_set_varname_fn bb_set_varname = bb_set_varname_default;
+bb_varname_max_len_fn bb_varname_max_len = bb_varname_max_len_default;
+
+#ifndef __KERNEL__
+static int32_t
+bb_varname_init(S_BB_T *bb)
+{
+  bb_varname_init_fn init;
+  bb_get_varname_fn getter;
+  bb_set_varname_fn setter;
+  bb_varname_max_len_fn max_len;
+  char getter_name[256];
+  char setter_name[256];
+  char init_name[256];
+  char max_len_name[256];
+  S_BB_PRIV_T *priv;
+  void *handle = RTLD_DEFAULT;
+
+  priv = bb_get_priv(bb);
+  if (priv->varname_lib[0] == '\0') {
+    return BB_OK;
+  }
+
+  sprintf(getter_name, "bb_get_varname_%s", priv->varname_lib);
+  sprintf(setter_name, "bb_set_varname_%s", priv->varname_lib);
+  sprintf(init_name, "bb_varname_init_%s", priv->varname_lib);
+  sprintf(max_len_name, "bb_varname_max_len_%s", priv->varname_lib);
+
+  getter = dlsym(handle, getter_name);
+  if (!getter) {
+    char libname[FILENAME_MAX];
+
+    sprintf(libname, "lib%s.so", priv->varname_lib);
+    /* try to open a library */
+    handle = dlopen(libname, RTLD_NOW);
+    if (!handle) {
+      bb_logMsg(BB_LOG_WARNING, "cound not find lib %s\n", libname);
+      return BB_NOK;
+    }
+    getter = dlsym(handle, getter_name);
+    if (!getter) {
+      bb_logMsg(BB_LOG_WARNING, "cound not find symbol %s\n", getter_name);
+      return BB_NOK;
+    }
+  }
+  setter = dlsym(handle, setter_name);
+  if (!setter) {
+    bb_logMsg(BB_LOG_WARNING, "cound not find symbol %s\n", setter_name);
+    return BB_NOK;
+  }
+
+  init = dlsym(handle, init_name);
+
+  if (init) {
+    if (init(bb) != BB_OK)
+      return BB_NOK;
+  }
+
+  max_len = dlsym(handle, max_len_name);
+  if (max_len)
+    bb_varname_max_len = max_len;
+
+  bb_set_varname = setter;
+  bb_get_varname = getter;
+
+  return BB_OK;
+}
+#else
+static int32_t
+bb_varname_init(S_BB_T *bb)
+{
+	return BB_OK;
+}
+#endif
+
+
+int32_t
+bb_ctl(S_BB_T *bb, unsigned int request, ...)
+{
+  va_list ap;
+  int ret=BB_OK;
+  char *name;
+  S_BB_PRIV_T *priv;
+
+  va_start(ap, request);
+  switch (request) {
+    /* We've been passed a function pointer */
+    case BB_CTL_SET_NAME_SETTER_PTR:
+      bb_set_varname = va_arg(ap, typeof(bb_set_varname));
+      break;
+      /* We've been passed a function pointer */
+    case BB_CTL_SET_NAME_GETTER_PTR:
+      bb_get_varname = va_arg(ap, typeof(bb_get_varname));
+      break;
+    case BB_CTL_GET_NAMING_FROM_BB:
+      ret=bb_varname_init(bb);
+      break;
+    case BB_CTL_SET_NAMING_IN_BB:
+      name = va_arg(ap, typeof(name));
+      priv = bb_get_priv(bb);
+      strncpy(priv->varname_lib, name, VARNAME_LIB_LENGTH);
+      ret=bb_varname_init(bb);
+      break;
+    default:
+      ret=BB_NOK;
+      break;
+  }
+  va_end(ap);
+
+  return ret;
+}
+
 int32_t
 bb_check_version(volatile S_BB_T* bb) {
   int32_t retval = 0;
@@ -191,6 +344,10 @@ bb_check_version(volatile S_BB_T* bb) {
 
   assert(bb);
   retval = BB_VERSION_ID - bb->bb_version_id;
+  /* From now on we're nice and backwards compatible */
+  if (BB_VERSION_ID >= 0x2000 && bb->bb_version_id >= 0x2000) {
+    return 0; 
+  }
 /*   if (retval != 0) { */
 /*       bb_logMsg(BB_LOG_WARNING, "BlackBoard::bb_check_version", */
 /* 		"BB version mismatch using <0x%08X> for accessing <0x%08X>\n", */
@@ -206,51 +363,62 @@ bb_size(const int32_t n_data, const int32_t data_size) {
    *  data descriptor array size +
    *  data zone size
    */
-  return (sizeof(S_BB_T) + 
+  int32_t size;
+
+  size = (sizeof(S_BB_T) + 
     sizeof(S_BB_DATADESC_T)*n_data +
     sizeof(char)*data_size);
+
+  size += sizeof(S_BB_PRIV_T);
+  return size;
 } /* end of bb_size */
 
-int32_t 
+S_BB_PRIV_T *
+bb_get_priv(volatile S_BB_T* bb)
+{
+  return (S_BB_PRIV_T *)((unsigned long)bb + (unsigned long)(bb_size(bb->max_data_desc_size, bb->max_data_size) - sizeof(S_BB_PRIV_T)));
+}
+
+int32_t
 bb_find(volatile S_BB_T* bb, const char* var_name) {
-  
-  int32_t retval;  
+  int32_t retval;
   int32_t i;
-    
+
   retval = -1;
   assert(bb);
 
   for (i=0; i< bb->n_data;++i) {
-    if (!strncmp(var_name,(bb_data_desc(bb)[i]).name,VARNAME_MAX_SIZE+1)) {
+    char * n = bb_get_varname(&bb_data_desc(bb)[i]);
+    if (!strcmp(var_name,n)) {
       retval = i;
+      free(n);
       break;
     }
+    free(n);
   } /* end for */
 
   return retval;
 } /* end of  bb_find */
 
-S_BB_DATADESC_T* 
+S_BB_DATADESC_T*
 bb_data_desc(volatile S_BB_T* bb) {
-  
-  S_BB_DATADESC_T* retval;  
-  
+  S_BB_DATADESC_T* retval;
+
   retval = NULL;
   assert(bb);
   retval = (S_BB_DATADESC_T*) ((char*)(bb) + bb->data_desc_offset);
-  
+
   return retval;
 } /* end of bb_data_desc */
 
-void* 
+void*
 bb_data(volatile S_BB_T* bb) {
-  
-  void* retval;  
-  
+  void* retval;
+
   retval = NULL;
   assert(bb);
   retval = (char*)(bb) + bb->data_offset;
-  
+
   return retval;
 } /* end of bb_data */
 
@@ -366,9 +534,9 @@ bb_data_initialise(volatile S_BB_T* bb, S_BB_DATADESC_T* data_desc,void* default
       break;
     case E_BB_USER:
       if (NULL == default_value) {
-	memset(data + (data_desc->type)*i,0,data_desc->type);
+				memset(data + (data_desc->type)*i,0,data_desc->type_size);
       } else {
-	memcpy(data + (data_desc->type)*i,default_value,data_desc->type);
+				memcpy(data + (data_desc->type)*i,default_value,data_desc->type_size);
       }
       break; 
     default:
@@ -489,15 +657,17 @@ int32_t
 bb_data_header_print(S_BB_DATADESC_T data_desc, FILE* pf, int32_t idx, int32_t aliastack) {
   const char oneTab[] = "    "; 
   char tabs[MAX_ALIAS_LEVEL*5]="";
-  
+  char *n;
   int i;
-  
+
   for (i=0; i<(aliastack-1); i++)
     {
-      strncat(tabs, oneTab, strlen(oneTab));
+      strncat(tabs, oneTab, sizeof(tabs)-strlen(tabs));
     }
-  
-  fprintf(pf,"%s---------- < %s > ----------\n", tabs,data_desc.name);
+
+  n = bb_get_varname(&data_desc);
+  fprintf(pf,"%s---------- < %s > ----------\n", tabs, n);
+  free(n);
   fprintf(pf,"%s  alias-target = %d\n", tabs, data_desc.alias_target);
   fprintf(pf,"%s  type         = %d  (%s)\n",tabs,data_desc.type,E_BB_2STRING[data_desc.type]);
   fprintf(pf,"%s  dimension    = %d  \n",tabs,data_desc.dimension);
@@ -538,7 +708,7 @@ bb_data_footer_print(S_BB_DATADESC_T data_desc, FILE* pf, int32_t idx, int32_t a
   int i;
   
   for (i=0; i<(aliastack-1); i++) {
-    strncat(tabs, oneTab, strlen(oneTab));
+    strncat(tabs, oneTab, strlen(oneTab)+sizeof(tabs));
   }
     
   if ((idx>=0) &&
@@ -817,59 +987,75 @@ bb_unlock(volatile S_BB_T* bb) {
   return ops[bb->type]->bb_unlock(bb);
 } /* end of bb_unlock */
 
-
 int32_t 
 bb_attach(S_BB_T** bb, const char* pc_bb_name) 
 {
   enum bb_type type;
+  int ret;
+
   type = bb_type(pc_bb_name);
-  return ops[type]->bb_shmem_attach(bb, pc_bb_name);
+  ret = ops[type]->bb_shmem_attach(bb, pc_bb_name);
+  if (ret != BB_OK)
+    return ret;
+
+  /* older versions don't support name encoding */
+  if ((*bb)->bb_version_id < 0x0004000)
+    return ret;
+
+#ifndef __KERNEL__
+  if (bb_varname_init(*bb) != BB_OK) {
+    bb_logMsg(BB_LOG_WARNING, "%s", "Could not setup a proper varname encoding scheme\n");
+  }
+#endif
+  return ret;
 } /* end of bb_attach */
 
-int32_t 
+int32_t
 bb_detach(S_BB_T** bb) {
-  
   assert(bb);
   assert(*bb);
-  
+
   return ops[(*bb)->type]->bb_shmem_detach(bb);
 } /* end of bb_detach */
 
-void* 
+void*
 bb_publish(volatile S_BB_T *bb, S_BB_DATADESC_T* data_desc) {
-  
   void* retval;
   int32_t needed_size;
+  char *n;
 
   retval = NULL;
   assert(bb);
   assert(data_desc);
-  
+
   /* Verify that the published data is not already published
    * (key unicity) and trigger automatic subscribe
    * if key already exists.
    */
   bb_lock(bb);
-  if (bb_find(bb,data_desc->name) != -1) {
-     bb_logMsg(BB_LOG_FINER,"BlackBoard::bb_publish",
-	       "Key <%s> already exists in blackboard (automatic subscribe)!!",data_desc->name);
+  n = bb_get_varname(data_desc);
+  if (bb_find(bb, n) != -1) {
+    char *name = bb_get_varname(data_desc);
+    bb_logMsg(BB_LOG_FINER,"BlackBoard::bb_publish",
+        "Key <%s> already exists in blackboard (automatic subscribe)!!", name);
+    free(name);
     bb_unlock(bb);
     retval = bb_subscribe(bb,data_desc);
     bb_lock(bb);
   } else {
     /* compute required data size  */
-    needed_size = data_desc->type_size*data_desc->dimension;    
+    needed_size = data_desc->type_size*data_desc->dimension;
     /* verify available space in BB data descriptor zone */
     if (bb->n_data >= bb->max_data_desc_size) {
-      bb_logMsg(BB_LOG_SEVERE,"BlackBoard::bb_publish", 
+      bb_logMsg(BB_LOG_SEVERE,"BlackBoard::bb_publish",
 		"No more room in BB data descriptor!! [current n_data=%d]",
 		bb->n_data);
       /* verify available space in BB data zone */
     } else if ((bb->max_data_size-bb->data_free_offset) < needed_size) {
-      bb_logMsg(BB_LOG_SEVERE,"BlackBoard::bb_publish", 
+      bb_logMsg(BB_LOG_SEVERE,"BlackBoard::bb_publish",
 		"No more room in BB data zone!! [left <%d> byte(s) out of <%d> required]",
 		bb->max_data_size-bb->data_free_offset,needed_size);
-    } else {     
+    } else {
       /* Compute the free address */
       retval = (char*) bb_data(bb) + bb->data_free_offset;
       /* Update returned data descriptor */
@@ -885,24 +1071,23 @@ bb_publish(volatile S_BB_T *bb, S_BB_DATADESC_T* data_desc) {
     }
     /* initialize publish data zone with default value */
     bb_data_initialise(bb,data_desc,NULL);
-  }    
-  /* no init in case of automatic subscribe */  
-  bb_unlock(bb);  
+  }
+  free(n);
+  /* no init in case of automatic subscribe */
+  bb_unlock(bb);
   return retval;
 } /* end of bb_publish */
 
-void* 
-bb_subscribe(volatile S_BB_T *bb, 
-	     S_BB_DATADESC_T* data_desc) {
-				 
-	int32_t          indexstack[MAX_ALIAS_LEVEL];
+void*
+bb_subscribe(volatile S_BB_T *bb, S_BB_DATADESC_T* data_desc) {
+	int32_t indexstack[MAX_ALIAS_LEVEL];
   	/* zero out indexstack */
 	memset(indexstack,0,MAX_ALIAS_LEVEL*sizeof(int32_t));
-	
+
 	return bb_alias_subscribe(bb,data_desc, indexstack,MAX_ALIAS_LEVEL);
 } /* end of bb_subscribe */
 
-void* 
+void*
 bb_item_offset(volatile S_BB_T *bb, 
 	       S_BB_DATADESC_T* data_desc,
 	       const int32_t* indexstack,
@@ -1173,41 +1358,47 @@ bb_get_array_name(char * array_name,
 #endif /* __KERNEL__ */
 
   indexstack_curr = 0;
-  for (j=aliasstack_size-1; j>=0; j--){
+  for (j=aliasstack_size-1; j>=0; j--) {
     /* If this alias is an array */
-    if (aliasstack[j].dimension > 1){
-      if (j==aliasstack_size-1){
-	snprintf(part_of_name, array_name_size_max, "%s[%0d]",
-		 aliasstack[j].name,
-		 indexstack[indexstack_curr]);
+    if (aliasstack[j].dimension > 1) {
+      if (j==aliasstack_size-1) {
+        char *n = bb_get_varname(&aliasstack[j]);
+        snprintf(part_of_name, array_name_size_max, "%s[%0d]", n,
+            indexstack[indexstack_curr]);
+        free(n);
       } else {
-	snprintf(part_of_name, array_name_size_max, "%s[%0d]",
-		 strstr(aliasstack[j].name,
-			aliasstack[j+1].name)+strlen(aliasstack[j+1].name),
-		 indexstack[indexstack_curr]);
+        char *n1, *n2;
+        n1 = bb_get_varname(&aliasstack[j]);
+        n2 = bb_get_varname(&aliasstack[j+1]);
+        snprintf(part_of_name, array_name_size_max, "%s[%0d]",
+            strstr(n1, n2 + strlen(n2)),
+            indexstack[indexstack_curr]);
+        free(n1);
+        free(n2);
       }      
-      strncat(array_name, part_of_name, array_name_size_max);
+      strncat(array_name, part_of_name, array_name_size_max-strlen(array_name));
       /* go to next index in the index stack */
       indexstack_curr++;
     }
     /* The current alias is a scalar */
     else {
+      char *n1, *n2;
       if (j==aliasstack_size-1){
-	snprintf(part_of_name, array_name_size_max, "%s", aliasstack[j].name);
+        n1 = bb_get_varname(&aliasstack[j]);
+        snprintf(part_of_name, array_name_size_max, "%s", n1);
+        free(n1);
       } else {
-	snprintf(part_of_name, array_name_size_max, "%s", 
-		 strstr(aliasstack[j].name, aliasstack[j+1].name) +
-		 strlen(aliasstack[j+1].name)
-		 );
+        n1 = bb_get_varname(&aliasstack[j]);
+        n2 = bb_get_varname(&aliasstack[j+1]);
+        snprintf(part_of_name, array_name_size_max, "%s",
+            strstr(n1, n2 + strlen(n2)));
+        free(n1);
+        free(n2);
       }
-      strncat(array_name, part_of_name, array_name_size_max);
+      strncat(array_name, part_of_name, array_name_size_max-strlen(array_name));
     }
   }
-#ifdef __KERNEL__
-  kfree (part_of_name);
-#else
   free (part_of_name);
-#endif
   return BB_OK;
 
 } /* end of get_array_name */
