@@ -1,6 +1,6 @@
 /*
 
-$Header: /cvsroot/tsp/tsp/src/util/libbb/bb_simple.c,v 1.11 2006/11/27 20:01:47 deweerdt Exp $
+$Header: /sources/tsp/tsp/src/util/libbb/bb_simple.c,v 1.16 2007/04/01 13:17:21 deweerdt Exp $
 
 -----------------------------------------------------------------------
 
@@ -57,43 +57,54 @@ pthread_mutex_t bb_simple_stcokage_mutex   = PTHREAD_MUTEX_INITIALIZER;
 #endif /* __KERNEL__ */
 
 static void gen_var_name(char *gen_var_name, const int module_instance,
-			 const char *module_name, const char *var_name)
+			                   const char *module_name, const char *var_name)
 {
-  memset(gen_var_name,0,VARNAME_MAX_SIZE); 
+  memset(gen_var_name,0,bb_varname_max_len()); 
   if (strcmp(module_name,"")==0) {
-    snprintf(gen_var_name,VARNAME_MAX_SIZE,
-	     "%s", var_name);
+    snprintf(gen_var_name,bb_varname_max_len(), "%s", var_name);
   } else if (-1!=module_instance) {
-    snprintf(gen_var_name,VARNAME_MAX_SIZE,
-	     "%s_%d_%s",
-	     module_name,module_instance,var_name);
+    snprintf(gen_var_name,bb_varname_max_len(), "%s_%d_%s",
+	           module_name,module_instance,var_name);
   } else {
-    snprintf(gen_var_name,VARNAME_MAX_SIZE,
-	     "%s_%s", module_name,var_name);
+    snprintf(gen_var_name,bb_varname_max_len(), "%s_%s", 
+             module_name,var_name);
   }
 }
 
 void* bb_simple_publish(S_BB_T* bb_simple,
-			const char* var_name, 
-			const char* module_name,
-			const int module_instance,
-			E_BB_TYPE_T type,
-			int type_size,
-			int dimension) {
-  
-  void* retval;
+    const char* var_name, 
+    const char* module_name,
+    const int module_instance,
+    E_BB_TYPE_T type,
+    int type_size,
+    int dimension) {
+
+  void* retval = NULL;
   S_BB_DATADESC_T s_data_desc;
-  gen_var_name(s_data_desc.name, module_instance, module_name, var_name);
+  char *new_name;
+  int ret;
+
+  new_name = malloc(sizeof(char)*bb_varname_max_len());
+  if (!new_name)
+    goto out;
+  gen_var_name(new_name, module_instance, module_name, var_name);
+  ret = bb_set_varname(&s_data_desc, new_name);
+  if (ret == BB_NOK)
+    goto err_free;
   s_data_desc.type        = type;
   s_data_desc.type_size   = type_size;
   s_data_desc.dimension   = dimension;
   retval = bb_publish(bb_simple,&s_data_desc);
 
   if (retval == NULL) {
-    bb_logMsg(BB_LOG_SEVERE,"BlackBoard::bb_simple_publish", 
-		"Cannot publish data <%s> instance <%d> module <%s>",
-		var_name,module_instance,module_name);
-  }  
+    bb_logMsg(BB_LOG_SEVERE,"BlackBoard::bb_simple_publish",
+        "Cannot publish data <%s> instance <%d> module <%s>",
+        var_name,module_instance,module_name);
+  }
+
+err_free:
+  free(new_name);
+out:
   return retval;
 } /* end of bb_simple_publish */
 
@@ -105,16 +116,26 @@ void* bb_simple_subscribe(S_BB_T* bb_simple,
 			  E_BB_TYPE_T*  type,
 			  int* type_size,
 			  int* dimension) {
-  
-  void* retval;
+  void* retval = NULL;
   S_BB_DATADESC_T s_data_desc;
+  char *new_name;
+	int ret;
 
-  gen_var_name(s_data_desc.name, module_instance, module_name, var_name);
-  
+  new_name = malloc(sizeof(char)*bb_varname_max_len());
+  if (!new_name)
+    goto out;
+  gen_var_name(new_name, module_instance, module_name, var_name);
+  ret = bb_set_varname(&s_data_desc, new_name);
+ 	if (ret == BB_NOK)
+		goto err_free;
   s_data_desc.type        = *type;
   s_data_desc.type_size   = *type_size;
   s_data_desc.dimension   = *dimension;
-  retval = bb_subscribe(bb_simple,&s_data_desc);  
+  retval = bb_subscribe(bb_simple,&s_data_desc);
+
+err_free:
+  free(new_name);
+out:
   return retval;
 } /* end of bb_simple_subscribe */
 
@@ -131,43 +152,62 @@ void* bb_simple_alias_publish(S_BB_T* bb_simple,
   
   void* retval=NULL;
   int32_t idx;
-  char new_target_name[VARNAME_MAX_SIZE];
+  char *new_target_name;
+  char *alias_name;
+	int ret;
   S_BB_DATADESC_T alias;
   S_BB_DATADESC_T target;
   
+  alias_name = malloc(sizeof(char)*bb_varname_max_len());
+  if (!alias_name)
+    goto out;
+  new_target_name = malloc(sizeof(char)*bb_varname_max_len());
+  if (!new_target_name)
+    goto err_free_1;
   gen_var_name(new_target_name, module_instance, module_name, target_name);
-  snprintf(alias.name, VARNAME_MAX_SIZE, "%s_%s",
+  snprintf(alias_name, bb_varname_max_len(), "%s_%s",
 	   new_target_name, var_name);
+  ret = bb_set_varname(&alias, alias_name);
+  if (ret == BB_NOK)
+    goto err_free_2;
 
   alias.type        = type;
   alias.type_size   = type_size;
   alias.dimension   = dimension;
   alias.data_offset = offset;
   
-  /* find the index of the target (it is the target for the aliases we want to publish) */                                                                                                                          
+  /* find the index of the target (it is the target for the aliases we want to publish) */
   idx = bb_find( bb_simple, new_target_name);
   if (idx ==-1) {
     bb_logMsg(BB_LOG_SEVERE,"BlackBoard::bb_simple_alias_publish", 
 		"Cannot find the target <%s> of the alias <%s>",
 		target_name, var_name);
-  }  
-  else {
-  		target = bb_data_desc(bb_simple)[idx];
+  } else {
+    char *n;
+		int ret;
+    target = bb_data_desc(bb_simple)[idx];
 
-  		memset(alias.name,0,VARNAME_MAX_SIZE); 
- 		snprintf(alias.name,VARNAME_MAX_SIZE,
-	    		 	"%s.%s",
-     		  		target.name,
-	  		   	var_name);
-		  
-  		retval=(void *)bb_alias_publish(bb_simple, &alias, &target);
-  
-  		if (retval == NULL) {
-    		bb_logMsg(BB_LOG_SEVERE,"BlackBoard::bb_simple_alias_publish", 
-						"Cannot publish data <%s> instance <%d> module <%s>",
-						var_name,module_instance,module_name);
-  		}
+    n = bb_get_varname(&target);
+    snprintf(alias_name,bb_varname_max_len(),
+        "%s.%s", n, var_name);
+    free(n);
+    ret = bb_set_varname(&alias, alias_name);
+		if (ret == BB_NOK)
+			goto err_free_2;
+
+    retval=(void *)bb_alias_publish(bb_simple, &alias, &target);
+
+    if (retval == NULL) {
+      bb_logMsg(BB_LOG_SEVERE,"BlackBoard::bb_simple_alias_publish", 
+          "Cannot publish data <%s> instance <%d> module <%s>",
+          var_name,module_instance,module_name);
+    }
   } 
+err_free_2:
+  free(new_target_name);
+err_free_1:
+  free(alias_name);
+out:
   return retval;
 } /* end of bb_simple_alias_publish */
 
