@@ -2,13 +2,17 @@
 #include <rpc/rpc.h>
 #include <pthread.h>
 #ifdef WIN32
-#  include <ws2tcpip.h>
-#  include "getopt.h" 
+    #include <ws2tcpip.h>
+    #include "getopt.h" 
 #else
-#  include <unistd.h>
-#  include <sys/types.h>
-#  include <sys/socket.h>
-#  include <netdb.h>
+    #include <unistd.h>
+    #include <sys/types.h>
+    #include <sys/socket.h>  
+    #include <netdb.h>
+#if defined(sun) || defined(__sun)
+    #include <strings.h>
+    #include <stdlib.h>
+#endif
 #endif
 #include "test_port_rpc.h"
 #include "test_api.h"
@@ -274,7 +278,7 @@ void rpc_svc_main(int argc, char* argv[])
     /* Main options handling */
     /*  char*   error_string;*/
     int     opt_ok  = 1;
-    char    c_opt;
+    int    c_opt;
     int    help     = 0;
     int    test_api = 0;
     int    test_rpc = 0;
